@@ -2,7 +2,7 @@ package RedisDB;
 
 use strict;
 use warnings;
-our $VERSION = "1.02_3";
+our $VERSION = "1.99_01";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
@@ -30,6 +30,11 @@ RedisDB - Perl extension to access redis database
 
 =head1 DESCRIPTION
 
+B<WARNING: this is a development version and is not recommended for production
+use.  This version implements XS version of the parser. Support for utf8 flag
+is not currently implemented and I'm in doubt if I should drop it altogether,
+if you're using it let me know.>
+
 This module provides interface to access redis key-value store. It
 transparently handles disconnects and forks. It supports transactions,
 pipelining, and subscription mode.
@@ -150,7 +155,7 @@ sub _connect {
     # this is to prevent recursion
     confess "Couldn't connect to the redis-server."
       . " Connection was immediately closed by the server."
-      if $self->{_in_connect}++;
+      if $self->{_in_connect};
 
     $self->{_pid} = $$;
 
@@ -182,6 +187,7 @@ sub _connect {
         };
     }
 
+    $self->{_in_connect}++;
     $self->_init_parser;
     $self->{_subscription_loop} = 0;
     delete $self->{_server_version};
@@ -533,16 +539,18 @@ sub version {
 my @commands = qw(
   append	auth	bgrewriteaof	bgsave	blpop	brpop   brpoplpush	config_get
   config_set	config_resetstat	dbsize	debug_object	debug_segfault
-  decr	decrby	del	echo	exists	expire	expireat	flushall
+  decr	decrby	del	echo	eval    evalsha exists	expire	expireat	flushall
   flushdb	get	getbit	getrange	getset	hdel	hexists	hget	hgetall
-  hincrby	hkeys	hlen	hmget	hmset	hset	hsetnx	hvals	incr	incrby
-  keys	lastsave	lindex	linsert	llen	lpop	lpush	lpushx
-  lrange	lrem	lset	ltrim	mget	move	mset	msetnx	persist	ping
+  hincrby	hincrbyfloat	hkeys	hlen	hmget	hmset	hset	hsetnx	hvals	incr	incrby
+  incrbyfloat	keys	lastsave	lindex	linsert	llen	lpop	lpush	lpushx
+  lrange	lrem	lset	ltrim	mget	move	mset	msetnx	persist
+  pexpire	pexpireat	psetex	pttl	ping
   publish	quit	randomkey	rename	renamenx	rpop	rpoplpush
-  rpush	rpushx	sadd	save	scard	sdiff	sdiffstore	select	set
+  rpush	rpushx	sadd	save	scard	script_exists   script_flush    script_kill
+  script_load   sdiff	sdiffstore	select	set
   setbit	setex	setnx	setrange	sinter	sinterstore
-  sismember	slaveof	smembers	smove	sort	spop	srandmember
-  srem	strlen	sunion	sunionstore	sync	ttl	type	unwatch watch
+  sismember	slaveof	slowlog smembers	smove	sort	spop	srandmember
+  srem	strlen	sunion	sunionstore	sync	time    ttl	type	unwatch watch
   zadd	zcard
   zcount	zincrby	zinterstore	zrange	zrangebyscore	zrank	zrem
   zremrangebyrank   zremrangebyscore	zrevrange	zrevrangebyscore	zrevrank
@@ -569,18 +577,20 @@ I<execute>, waits for the reply from the server, and returns it. E.g.:
     $redis->send_command("get", $key, sub { $val = $_[1] });
 
 The following wrapper methods are defined: append, auth, bgrewriteaof, bgsave,
-blpop, brpop, brpoplpush, config_get, config_set, config_resetstat, dbsize,
-debug_object, debug_segfault, decr, decrby, del, echo, exists, expire,
-expireat, flushall, flushdb, get, getbit, getrange, getset, hdel, hexists,
-hget, hgetall, hincrby, hkeys, hlen, hmget, hmset, hset, hsetnx, hvals, incr,
-incrby, keys, lastsave, lindex, linsert, llen, lpop, lpush, lpushx, lrange,
-lrem, lset, ltrim, mget, move, mset, msetnx, persist, ping, publish, quit,
+blpop, brpop,   brpoplpush, config_get, config_set, config_resetstat, dbsize,
+debug_object, debug_segfault, decr, decrby, del, echo, eval,    evalsha,
+exists, expire, expireat, flushall, flushdb, get, getbit, getrange, getset,
+hdel, hexists, hget, hgetall, hincrby, hincrbyfloat, hkeys, hlen, hmget, hmset,
+hset, hsetnx, hvals, incr, incrby, incrbyfloat, keys, lastsave, lindex,
+linsert, llen, lpop, lpush, lpushx, lrange, lrem, lset, ltrim, mget, move,
+mset, msetnx, persist, pexpire, pexpireat, psetex, pttl, ping, publish, quit,
 randomkey, rename, renamenx, rpop, rpoplpush, rpush, rpushx, sadd, save, scard,
-sdiff, sdiffstore, select, set, setbit, setex, setnx, setrange, sinter,
-sinterstore, sismember, slaveof, smembers, smove, sort, spop, srandmember,
-srem, strlen, sunion, sunionstore, sync, ttl, type, unwatch, watch, zadd,
+script_exists,   script_flush,    script_kill, script_load,   sdiff,
+sdiffstore, select, set, setbit, setex, setnx, setrange, sinter, sinterstore,
+sismember, slaveof, slowlog, smembers, smove, sort, spop, srandmember, srem,
+strlen, sunion, sunionstore, sync, time,    ttl, type, unwatch, watch, zadd,
 zcard, zcount, zincrby, zinterstore, zrange, zrangebyscore, zrank, zrem,
-zremrangebyrank, zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
+zremrangebyrank,   zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
 zscore, zunionstore
 
 See description of all commands in redis documentation at
@@ -589,16 +599,15 @@ L<http://redis.io/commands>.
 =cut
 
 for my $command (@commands) {
-    my $uccom = uc $command;
-    $uccom =~ s/_/ /g;
+    my @uccom = split /_/, uc $command;
     no strict 'refs';
     *{ __PACKAGE__ . "::$command" } = sub {
         my $self = shift;
         if ( ref $_[-1] eq 'CODE' ) {
-            return $self->send_command( $uccom, @_ );
+            return $self->send_command( @uccom, @_ );
         }
         else {
-            return $self->execute( $uccom, @_ );
+            return $self->execute( @uccom, @_ );
         }
     };
 }
@@ -634,7 +643,7 @@ answer.  Croaks in case of the error.
 
 sub shutdown {
     my $self = shift;
-    $self->send_command_cb('SHUTDOWN');
+    $self->send_command_cb('SHUTDOWN', @_);
     return;
 }
 
@@ -1067,9 +1076,13 @@ Known bugs are:
 Timeout support is OS dependent. If OS doesn't support SO_SNDTIMEO and SO_RCVTIMEO
 options timeouts will not work.
 
+QUIT command doesn't work with redis-server before version 2.0
+
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to Sanko Robinson and FunkyMonk for help with porting this module on Windows.
+Sanko Robinson and FunkyMonk helped me with porting this module to Windows.
+
+HIROSE Masaake fixed handling of commands containing space (like "CONFIG GET")
 
 =head1 AUTHOR
 
